from green import *
import cmath
import harmonique
import matplotlib.pyplot as plt

# Configuration
perturb = 1e-2
N=5
npts=32
cl_int = 0.
cl_ext = 1.
centres = [0., 2./5.] # Le premier cercle est ext\'erieur
rayons = [1., 2./5.]
if (len(centres) != len(rayons)):
    print( "Verifier nombre de cercles")
for cen, ray in zip(centres[1:], rayons[1:]):
    if (abs(cen-centres[0])+ray > rayons[0]):
        print( "Verifier que le premier cercle est exterieur aux autres")

# solution exacte
potentiel_harmonique = harmonique.solution_couronne(rayons[0], rayons[1], centres[1])

def gradient_harmonique(z, pas=1e-4):
    """gradient de la solution exacte sur le domain initial
    """
    gnx = (potentiel_harmonique(z+pas) - potentiel_harmonique(z-pas)) /2/pas
    gny = (potentiel_harmonique(z+1j*pas) - potentiel_harmonique(z-1j*pas)) /2/pas
    return gnx, gny
    

# Perturbation du domaine
centres_pert = list(centres)
rayons_pert = list(rayons)
i_pert = 1

centres_pert[i_pert] = centres[i_pert] + perturb

def delta_rho(t):
    nx = -math.cos(t)
    ny = -math.sin(t)
    px = perturb.real
    py = perturb.imag
    res = px*nx + py*ny
    return res

potentiel_harmonique_pert = harmonique.solution_couronne(rayons_pert[0], rayons_pert[1], centres_pert[1])


# Grille de points dans le domaine perturb\'e
xvals = np.arange(-1., 1., .01)
xpos = [x+1j*y for x in xvals for y in xvals
        if point_interieur(x+1j*y, centres_pert, rayons_pert,eps=1e-3)]

# Norme 1 de l'erreur relative
res = []
for zeta in xpos:
    # solutions sur le domaine initial
    g_zeta, gradg_zeta = green(centres, rayons, zeta, npts, N)
    u1 = potentiel_harmonique(zeta)
    
    # delta u en zeta
    delta_u = eval_delta_u(zeta,
                           centres[i_pert], rayons[i_pert],
                           delta_rho,
                           gradient_harmonique,
                           gradg_zeta)
    
    res.append(abs(delta_u)/perturb)

# Tracer le potentiel
xres = [e.real for e in xpos]
yres = [e.imag for e in xpos]

plt.clf()
sc = plt.scatter(xres, yres, c=res,edgecolors='none')
plt.colorbar(sc)
plt.gca().set_aspect('equal', adjustable='box')
plt.title("Variation de hadamard")
plt.show()
